import { useMemo } from 'react';
import { useUnmount } from './useUnmount';
class AsyncInstance {
    constructor() {
        Object.defineProperty(this, "clearFns", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
    }
    clearAll() {
        for (const clear of this.clearFns) {
            clear();
        }
    }
    setTimeout(handler, timeout) {
        const tid = window.setTimeout(handler, timeout);
        const clear = () => {
            clearTimeout(tid);
        };
        this.clearFns.add(clear);
        return clear;
    }
    requestAnimationFrame(...args) {
        const tid = requestAnimationFrame(...args);
        const clear = () => {
            cancelAnimationFrame(tid);
        };
        this.clearFns.add(clear);
        return clear;
    }
}
export class Async extends AsyncInstance {
    constructor() {
        super(...arguments);
        Object.defineProperty(this, "instances", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
    }
    create() {
        const instance = new AsyncInstance();
        this.instances.add(instance);
        return instance;
    }
    clearAll() {
        for (const clear of this.clearFns) {
            clear();
        }
        for (const instance of this.instances) {
            instance.clearAll();
        }
    }
}
export function useAsync() {
    const async = useMemo(() => new Async(), []);
    useUnmount(() => {
        async.clearAll();
    });
    return async;
}
